package ch.zhaw.catan;

import ch.zhaw.catan.Config.Land;
import ch.zhaw.hexboard.HexBoard;
import ch.zhaw.hexboard.Label;

import java.awt.*;
import java.util.List;
import java.util.*;

public class SiedlerBoard extends HexBoard<Land, Settlement, Road, String> {

    Map<Point, Field> fields = new HashMap<>();

    public void createFixGamefield(){
        Map<Point,Land> resourcePlacement = Config.getStandardLandPlacement();
        Map<Point, Integer> dicePlacement = Config.getStandardDiceNumberPlacement();
        for (Map.Entry<Point,Land> resourceField : resourcePlacement.entrySet()) {
            addField(resourceField.getKey(),resourceField.getValue());
            if(dicePlacement.get(resourceField.getKey()) != null){
                String numberAsString = dicePlacement.get(resourceField.getKey()).toString();
                char[] numbersInChar = numberAsString.toCharArray();
                if (numberAsString.length() < 2) {
                    fields.put(resourceField.getKey(), new Field(resourceField.getValue(), new Label('0', numbersInChar[0])));
                }
                else {
                    fields.put(resourceField.getKey(), new Field(resourceField.getValue(), new Label(numbersInChar[0],numbersInChar[1])));
                }
            }
        }
    }


    private int getDiceNumber(Point field) {
        Label label = fields.get(field).getLabel();
        return Integer.parseInt(label.toString());
    }


    public String getTextView () {
        SiedlerBoardTextView view = new SiedlerBoardTextView(this);
        for (Map.Entry<Point, Field> field : fields.entrySet()) {
            view.setLowerFieldLabel(field.getKey(), field.getValue().getLabel());
        }
        return view.toString();
    }

    //TODO: Add fields, constructors and methods as you see fit. Do NOT change the signature
    //      of the methods below.

    /**
     * Returns the fields associated with the specified dice value.
     *
     * @param dice the dice value
     * @return the fields associated with the dice value
     */
    public List<Point> getFieldsForDiceValue(int dice) {
        ArrayList<Point> fields = new ArrayList<>();
        for(Point field : this.fields.keySet()){
            if(getDiceNumber(field) == dice){
               fields.add(field);
            }
        }
        return fields;
    }


    public ArrayList<Config.Resource> getResourcesforFaction(Point point, Config.Faction faction){
        List <Settlement> possibleSettlementField = super.getCornersOfField(point);
        ArrayList<Config.Resource> resourcesToPlayer = new ArrayList<>();
        for (Structure structure : possibleSettlementField) {
            if (structure.getFaction() == faction) {
                resourcesToPlayer.add(fields.get(point).getResource());
                if (structure instanceof City) {
                    resourcesToPlayer.add(fields.get(point).getResource());
                }
            }
        }
        return resourcesToPlayer;
    }

    /**
     * Returns the {@link Land}s adjacent to the specified corner.
     *
     * @param corner the corner
     * @return the list with the adjacent {@link Land}s
     */
    public List<Land> getLandsForCorner(Point corner) {
        Point above = new Point(corner.x, corner.y + 2);
        Point below = new Point(corner.x, corner.y -2);
        Land[] lands = new Land[3];
        if (hasField(above)) {
            lands[0] = getField(above);
            lands[1] = getField(new Point(corner.x + 1, corner.y - 1));
            lands[2] = getField(new Point(corner.x - 1, corner.y - 1));
        }
        else if (hasField(below)) {
            lands[0] = getField(below);
            lands[1] = getField(new Point(corner.x + 1, corner.y + 1));
            lands[2] = getField(new Point(corner.x - 1, corner.y + 1));
        }
        else {
            return Collections.emptyList();
        }
        return List.of(lands);
    }
}
